/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIESREGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2011, 2016 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/*******************************************************************************
* File Name    : UART0.c
* Device(s)    : R5F100LE
* Tool-Chain   : CCRL
* Description  : リングバッファのサポート関数です
* Creation Date: 2016/08/11
*******************************************************************************/

/*******************************************************************************
Includes
*******************************************************************************/
#include "r_cg_macrodriver.h"
#include "UART0.h"
#include "r_cg_userdefine.h"

/*******************************************************************************
Pragma directive
*******************************************************************************/

/*******************************************************************************
Global variables and functions
*******************************************************************************/
volatile uint8_t g_rx_buff[16];                 /* 受信用リングバッファ     */
volatile uint8_t g_rx_rdpt;                     /* データの読み出しポインタ */
volatile uint8_t g_rx_dtno;                     /* 格納されているデータ数   */
volatile uint8_t g_rx_status;                   /* 受信ステータスフラグ     
        bit7 : オーバフロー・フラグ                                         */                                                                                                                          
/*******************************************************************************
* Function Name: init_bf
* Description  : リングバッファのポインタを初期化
* Arguments    : None
* Return Value : None
*******************************************************************************/
void init_bf(void)
{

    SRMK0 = 1;                                  /* INTSR0との排他制御       */
    g_rx_rdpt = 0x00;                           /* 読み出しポインタ初期化   */
    g_rx_dtno = 0x00;                           /* データ数をクリア         */
    g_rx_status = 0x00;                         /* ステータスをクリア       */
    SRMK0 = 0;                                  /* 排他制御完了             */

}

/*******************************************************************************
* Function Name: chk_status
* Description  : リングバッファの状態確認
* Arguments    : None
* Return Value : 受信データ数+ステータス
*               bit7 : オーバフロー・フラグ
*               bit6,5 : 0,0
*               bit4-0 : データ数
*******************************************************************************/
uint8_t chk_status(void)
{
    uint8_t work;

    SRMK0 = 1;                                  /* INTSR0との排他制御       */
    work = g_rx_status;                         /* ステータス読み出し       */
    g_rx_status = 0x00;                         /* ステータスをクリア       */
    work |= g_rx_dtno;                          /* データ数読み出し         */
    SRMK0 = 0;                                  /* 排他制御完了             */
    return ( work );

}

/*******************************************************************************
* Function Name: get_data
* Description  : リングバッファから1データを読み出す
* Arguments    : None
* Return Value : データ
*******************************************************************************/
uint8_t get_data(void)
{
    uint8_t work;

    if ( 0 == g_rx_dtno )
    {
        work = 0x00;                            /* データがなければ，0x00   */
    }
    else
    {
        work = g_rx_buff[g_rx_rdpt];            /* データの読み出し         */

        SRMK0 = 1;                              /* INTSR0との排他制御       */
        g_rx_dtno--;
        g_rx_rdpt++;                            /* ポインタ更新             */
        g_rx_rdpt &= 0x0F;
        SRMK0 = 0;                              /* 排他制御完了             */
    }

    return ( work );

}

/*******************************************************************************
* Function Name: get_blk
* Description  : リングバッファから指定されたデータを読み出す（転送）
* Arguments    : 格納ポインタ，データ数
* Return Value : 残りデータ数
*******************************************************************************/
uint8_t get_blk(uint8_t * const buff, uint8_t number)
{

    uint8_t work;
    uint8_t cnt;                                /* 転送用カウンタ           */
    uint8_t * gp_buff;                          /* 転送用ポインタ           */

    work = number;                              /* 要求データ数をセット     */

    if ( ( 0 != number ) && ( 0 != g_rx_dtno ) )
    {                               /* 転送データがある場合                 */

        gp_buff = buff;                         /* 転送先ポインタを設定     */

/* ---------------------------------------------------------------------------

    転送データ数を算出

----------------------------------------------------------------------------*/

        if ( number < g_rx_dtno )
        {                           /* 要求数がデータ量以下の場合           */
            cnt = number;                       /* 転送データ数をセット     */
            work = 0;                           /* 残りデータ数は0にする    */
        }
        else
        {                           /* 要求数がデータ量以上の場合           */
            cnt = g_rx_dtno;                    /* 転送数はバッファ内の全て */
            work = number - g_rx_dtno;          /* 残りデータ数を算出       */
        }

/* ---------------------------------------------------------------------------

    データを転送

----------------------------------------------------------------------------*/

        for (  ; cnt > 0 ; cnt-- )
        {
            *gp_buff = g_rx_buff[g_rx_rdpt];    /* リングバッファから転送   */
            SRMK0 = 1;                          /* INTSR0との排他制御       */
            g_rx_rdpt++;                        /* 読み出しポインタを更新   */
            g_rx_rdpt &= 0x0F;
            g_rx_dtno--;                        /* データ数を－1            */
            SRMK0 = 0;                          /* INTSR0との排他制御終了   */
            gp_buff++;                          /* 転送ポインタを更新       */

        }
    

    }

    return( work );

}
