/* FreeRTOS related resources. */
#include "freertos_start.h"

/* Eval board specific definitions. */
#include "demo_specific_io.h"

/* Other includes. */
#include "r_cg_intc.h"
#include "r_cg_timer.h"
#include "r_cg_adc.h"
#include "r_cg_dmac.h"

/* NOTE: Renesas FreeRTOS project does not support the main() function.
When you need it, try to use the Processing_Before_Start_Kernel() function
in the src/frtos_startup/freertos_start.c instead of it. */

void main_task(void *pvParameters)
{
    (void) pvParameters;

    while (1)
    {
        taskENTER_CRITICAL();
        P1_bit.no6 = !P1_bit.no6;
        taskEXIT_CRITICAL();
    }
}

void second_task(void *pvParameters)
{
    (void) pvParameters;

    while (1)
    {
        taskENTER_CRITICAL();
        P1_bit.no5 = !P1_bit.no5;
        taskEXIT_CRITICAL();
    }
}

void third_task(void *pvParameters)
{
    (void) pvParameters;

    while (1)
    {
        taskENTER_CRITICAL();
        P1_bit.no4 = !P1_bit.no4;
        taskEXIT_CRITICAL();
    }
}

volatile uint8_t g_AdResult[1024];      /* demo uses [0]...[99]. */

void intp_timer_adc_dmac_demo_task(void *pvParameters)
{
    uint8_t count;
    
    (void) pvParameters;
    NOP();  /* for breakpoint, start singnal input on Simulator GUI */

    taskENTER_CRITICAL();
    LED_INIT();
    taskEXIT_CRITICAL();

    R_INTC3_Start();

    R_ADC_Set_OperationOn();
    for (count = 0U ; count < 32U ; count++)    /* Waiting for comparator stabilizing */
    {
        NOP();
    }

    for (;;)
    {
        /* --- Waiting for a click of button on Simulator GUI --- */
        xSemaphoreTake( xSemaphoreINTP3, portMAX_DELAY );

        taskENTER_CRITICAL();
        LED_BIT = !LED_BIT;
        taskEXIT_CRITICAL();

        R_DMAC0_UserInit();

        /* --- Start TAU and A/D Convert and DMA --- */
        R_DMAC0_Start();                        /* Enable DMA : Start waiting DMA trigger */
        R_ADC_Start();                          /* Enable A/D : Start waiting A/D trigger */
        R_TAU0_Channel1_Lower8bits_Start();     /* Start! */

        /* --- A/D converting now --- */
        xSemaphoreTake( xSemaphoreINTDMA0, portMAX_DELAY );

        /* --- Stop TAU and A/D Convert and DMA --- */
        R_TAU0_Channel1_Lower8bits_Stop();
        R_ADC_Stop();
        R_DMAC0_Stop();

        taskENTER_CRITICAL();
        LED_BIT = !LED_BIT;
        taskEXIT_CRITICAL();

        NOP();  /* for trace start, check timing chart on Simulator GUI after trace full break */
    }
}
